/* Shaderprogramozs - Grafika s jtkfejleszts DirectX-szel ----------------

	llomny          : CPrimitive.cpp

    Nyelv             : C++

    Ltrehozs dtuma : 2008.08.28

    Utols mdosts  : 2009.03.15

    Lers            : Shaderprogramozs
						Grafika s jtkfejleszts DirectX-szel
						11.1.4. "A CPrimitive osztly"
						264. oldal

               Copyright (C) Nyisztor Kroly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/
/*- Szksges fggvnyknyvtrak --------------------------------------------*/
// biztostsuk a szksges fggvnyknyvtrak linkelst!

/*- Befordtott fejllomnyok -----------------------------------------------*/
#include "CPrimitive.h"

#include "..\..\common\util\util.h"
using namespace app::util;

#include "..\..\common\util\core.h"
using app::util::Core;


/*- Nvterek ----------------------------------------------------------------*/
namespace app
{
namespace scene
{
/*- Defincik s konstansok ------------------------------------------------*/

/*- Globlis vltozk -------------------------------------------------------*/

/*- Implementcis rsz -----------------------------------------------------*/

CPrimitive::CPrimitive( const string& fxFileName_in,
						const D3DXVECTOR3& posVec_in, 
						const D3DXVECTOR3& rotVec_in, 
						const D3DXVECTOR3& scaleVec_in ) : myFXFile( fxFileName_in )
{
	myFaultFlag = true;
	// D3DDevice mutat lekrdezse
	myD3DDevicePtr = GameCore->Device();

	myTriangleListVBPtr = NULL;
	myEffectPtr = NULL;
	// vertex adatformtum definilsa
	initVertexDeclaration();
	// vertex puffer feltltse
	initVertexBuffers();
	// effekt betltse
	CHECK_WEXIT( loadFX() );

	char messStr[256];
	sprintf_s( messStr,"Sikeres effect betlts s inicializls. Fjl: %s", fxFileName_in.c_str() );
	LOG_INFO( messStr )
}

CPrimitive::~CPrimitive()
{
	SAFE_RELEASE( myTriangleListVBPtr );
	SAFE_RELEASE( myEffectPtr );
	SAFE_RELEASE( myDeclPtr );
	
}
/*!
 * \brief Vertex-struktra megadsa
 * \remarks a shader ez alapjn kezeli a cscspontokat
 */
HRESULT CPrimitive::initVertexDeclaration()
{
	//3D-koordinta vertex struktra
	D3DVERTEXELEMENT9 vertexElements[] = 
	{
		{0, 0,  D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		D3DDECL_END()
	};
	CHECK_WEXIT( myD3DDevicePtr->CreateVertexDeclaration( vertexElements, &myDeclPtr) );

	LOG_INFO( "Vertex deklarls rendben" );
	return S_OK;
}

HRESULT CPrimitive::initVertexBuffers()
{
	// ez a pointer a vertexpuffer adatterletre fog mutatni
	SPrimitiveVertex* vertPtr = NULL;
	// 1) hromszg lista
	SPrimitiveVertex triangleList[] =
	{
		D3DXVECTOR3( -0.5f, 0.0f, 1.0f ), // hromszg cscspontjai
		D3DXVECTOR3(  0.0f, 0.5f, 1.0f ),
		D3DXVECTOR3(  0.5f, 0.0f, 1.0f )
	};

	CHECK_WEXIT(myD3DDevicePtr->CreateVertexBuffer( 3 * sizeof( SPrimitiveVertex ),
													0, 
													0,
													D3DPOOL_DEFAULT, 
													&myTriangleListVBPtr,
													NULL ) );
	// megszerezzk majd zroljuk a vertex puffer memriaterlett, hogy tmsolhassuk a vertex adatokat
	CHECK_WEXIT( myTriangleListVBPtr->Lock( 0, sizeof( triangleList ), (VOID**)&vertPtr, 0 ) );

	// vertex adatok msolsa
	memcpy( vertPtr, triangleList, sizeof( triangleList ) );

	// vertex puffer zrolsnak feloldsa
	CHECK_WEXIT( myTriangleListVBPtr->Unlock() );
	LOG_INFO("Vertex buffer init sikeres!")

	return S_OK;
}

HRESULT CPrimitive::loadFX()
{
	// ltezik a fjl? - megprbljuk megnyitni
	CHECK_WEXIT( fileExists(myFXFile) )
	
	// effekt ltrehozsa fjbl
	ID3DXBuffer* errorBufferPtr = NULL;
	// kritikus hvs
	CHECK_WEXIT( D3DXCreateEffectFromFile( myD3DDevicePtr, 
							  myFXFile.c_str(), 
							  0, 
							  0, 
							  0,/*D3DXSHADER_DEBUG, */
							  0, 
							  &myEffectPtr, 
							  &errorBufferPtr
							));
	// effekt betltsi hiba
	if( errorBufferPtr )
	{
		MessageBox(0, (char*)errorBufferPtr->GetBufferPointer(), 0, 0);
		LOG_ERROR( (char*)errorBufferPtr->GetBufferPointer() );
		return E_FAIL;
	}

	// kezelk lekrdezse
	myTechHandle       = myEffectPtr->GetTechniqueByName("SimpleTech");
	myWVPHandle        = myEffectPtr->GetParameterByName(0, "gWorldViewProjection");
	if( !myTechHandle
		|| !myWVPHandle )
	{
		return E_FAIL;
	}
	LOG_INFO( "Sikeres effekt betlts" );
	return S_OK;
}

void CPrimitive::render()
{	
	myD3DDevicePtr->SetStreamSource( 0, myTriangleListVBPtr, 0, sizeof(SPrimitiveVertex) );
	myD3DDevicePtr->SetVertexDeclaration(myDeclPtr);
	
	
	myEffectPtr->SetMatrix(myWVPHandle, &( /*myWorldMat * */ GameCore->Camera()->getViewProjMatrix() ));
	CHECK_WEXIT( myEffectPtr->SetTechnique(myTechHandle) );
	UINT passCnt = 0;
	// technika elkezdse, itercik szmnak lekrse
	myEffectPtr->Begin( &passCnt, 0 );
	for( UINT p = 0; p < passCnt; ++p )
	{
		// lps elkezdse
		myEffectPtr->BeginPass( p );
		// geometria kirajzolsa
		myD3DDevicePtr->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 1 );
		// lps vge
		myEffectPtr->EndPass();
	}
	// technika vge
	myEffectPtr->End();
}

void CPrimitive::onLostDevice()
{
	myEffectPtr->OnLostDevice();
	SAFE_RELEASE( myTriangleListVBPtr );
}

void CPrimitive::onResetDevice()
{
	initVertexBuffers();
	myEffectPtr->OnResetDevice();
}

}
}

