﻿/*[ Fordítási egység ----------------------------------------------------------

   Név				 : CInput.cpp

   Nyelv			 : C++

   Létrehozás dátuma : 2008.10.17

   Utolsó módosítás  : 2008.10.17

   Leírás			 : Ez az állomány a CInput osztály implementációját 
					   tartalmazza

			   Copyright (C) Nyisztor Károly 2008-2009

-----------------------------------------------------------------------------*/

/*- Szükséges függvénykönyvtárak --------------------------------------------*/
#pragma comment( lib, "dinput8.lib" )

/*- Befordított fejállományok -----------------------------------------------*/

#include "..\util\core.h"
using namespace app::util;
#include "CInput.h"
/*- Névterek ----------------------------------------------------------------*/
namespace app
{
namespace input
{

/*- Definíciók és konstansok ------------------------------------------------*/

/*- Globális változók -------------------------------------------------------*/

/*- Implementációs rész -----------------------------------------------------*/
/*!
 * \brief
 * gyártófüggvény a singleton CInput osztályhoz
 * létrehozza az egyedüli CInput példányt
 */
CInput* CInput::Instance()
{
	static CInput instance( GameCore->getHWND(), 
							GameCore->getAppInstance(), 
							DISCL_NONEXCLUSIVE|DISCL_BACKGROUND, 
							DISCL_NONEXCLUSIVE|DISCL_BACKGROUND);     
   static bool myCreatedFlag = false;
   if ( !myCreatedFlag )
   {
		myCreatedFlag = true;
   }
   
   return &instance;
}
/*!
 * \brief
 * DirectInput, egér és billentyűzet interfészek inicializálása
 *
 * \remarks
 * bővebben: 281-284 oldal
 */
CInput::CInput( HWND winHandle, HINSTANCE appInstance, DWORD keyboardFlags, DWORD mouseFlags )
{
	ZeroMemory(myKeyboardState, sizeof(myKeyboardState));
	ZeroMemory(&myMouseState, sizeof(myMouseState));

	CHECK_WEXIT(DirectInput8Create( appInstance, // alkalmazás példánya
						   DIRECTINPUT_VERSION, 	// DirectInput verzió
						   IID_IDirectInput8, 		// egyedi interfész azonosító
						   (void**)&myDirectInputPtr, // létrehozandó DirectInput pointerünk címe
						   0 ) );					// COM-mal kapcsolatos paraméter, maradhat NULL
	// billentyûzet objektum létrehozása
	CHECK_WEXIT(myDirectInputPtr->CreateDevice(GUID_SysKeyboard, &myKeyboardPtr, 0));
	// billentyűzet adatformátumának beállítása
	CHECK_WEXIT(myKeyboardPtr->SetDataFormat(&c_dfDIKeyboard));
	// bilentyűzet együttműködési szintjének beállítása
	CHECK_WEXIT(myKeyboardPtr->SetCooperativeLevel(winHandle, keyboardFlags));
	// billentyűzet figyelése
	CHECK_WEXIT(myKeyboardPtr->Acquire());
	// egér objektum létrehozása
	CHECK_WEXIT(myDirectInputPtr->CreateDevice(GUID_SysMouse, &myMousePtr, 0));
	// egér adatformátumának beállítása
	CHECK_WEXIT(myMousePtr->SetDataFormat(&c_dfDIMouse2));
	// egér együttműködési szintjének beállítása
	CHECK_WEXIT(myMousePtr->SetCooperativeLevel(winHandle, mouseFlags));
	// egér figyelése
	CHECK_WEXIT(myMousePtr->Acquire());
	
	LOG_INFO( "Sikeres egér és billentyûzet inicializálás." );
}
/*!
 * \brief
 * erőforrások felszabadítása
 *
 * \remarks
 * bővebben: 286 oldal
 */
CInput::~CInput()
{
	SAFE_RELEASE(myDirectInputPtr);
	myKeyboardPtr->Unacquire();
	myMousePtr->Unacquire();
	SAFE_RELEASE(myKeyboardPtr);
	SAFE_RELEASE(myMousePtr);
}
/*!
 * \brief
 * a billentyűzet és az egér lekérdezése
 *
 * \remarks
 * bővebben: 285-286 oldal
 */
void CInput::update()
{
	// billentyűzet állapotának lekérdezése
	HRESULT hr = myKeyboardPtr->GetDeviceState(sizeof(myKeyboardState), (void**)&myKeyboardState); 
	// hiba: szükséges a billentyűzet újrainicializálása 
	if( FAILED(hr) )
	{
		ZeroMemory(myKeyboardState, sizeof(myKeyboardState));
		// kapcsolat létesítése a billentyűzettel
		myKeyboardPtr->Acquire();
	}

	// egér állapotának lekérdezése
	hr = myMousePtr->GetDeviceState(sizeof(DIMOUSESTATE2), (void**)&myMouseState); 
	// hiba: szükséges az egér újrainicializálása 
	if( FAILED(hr) )
	{		
		ZeroMemory(&myMouseState, sizeof(myMouseState));
		// kapcsolat létesítése az egérrel
		myMousePtr->Acquire(); 
	}
}
/*!
 * \brief
 * billentyűk állapotának vizsgálata
 * \remarks
 * bővebben: 285. oldal
 */
bool CInput::isKeyDown(char key)
{
	return (myKeyboardState[key] & 0x80) != 0;
}
/*!
 * \brief
 * egérbillentyűk állapotának vizsgálata
 * 
 * \param button
 * gomb sorszáma (bal=0, jobb=1, középső=2, )
 */
bool CInput::isMouseButtonDown(int button)
{
	return (myMouseState.rgbButtons[button] & 0x80) != 0;
}
/*!
 * \brief
 * az egér x-tengely mentén történő elmozdulása
 * \remarks
 * bővebben: 286. oldal
 */
float CInput::getMouseDX()
{
	return (float)myMouseState.lX;
}
/*!
 * \brief
 * az egér y-tengely mentén történő elmozdulása
 * \remarks
 * bővebben: 286. oldal
 */
float CInput::getMouseDY()
{
	return (float)myMouseState.lY;
}
/*!
 * \brief
 * a görgő mozgásának változásait adja vissza
 * \remarks
 * bővebben: 286. oldal
 */
float CInput::getMouseDZ()
{
	return (float)myMouseState.lZ;
}

} // namespace input
} // namespace app
